package models

import (
	"encoding/json"
	"eta_gn/eta_index_lib/global"
	"eta_gn/eta_index_lib/utils"
	"fmt"
	"time"
)

type EdbInfoReq struct {
	EdbInfoId int `description:"图表id,新增时传0"`
}

// DeleteChartInfoDataRedis 清除图表缓存
func DeleteChartInfoDataRedis(bodyByte []byte) (err error) {
	var req EdbInfoReq
	err = json.Unmarshal(bodyByte, &req)
	if err != nil {
		return
	}
	if req.EdbInfoId > 0 {
		list, tmpErr := GetChartEdbMappingListByEdbInfoId(req.EdbInfoId)
		if tmpErr != nil {
			err = tmpErr
			return
		}

		// 删除图表的id
		for _, v := range list {
			_ = utils.Rc.Delete(GetChartInfoDataKey(v.ChartInfoId))
		}
	}
	return
}

// GetChartInfoDataKey 获取图表缓存的key
func GetChartInfoDataKey(chartInfoId int) string {
	key := fmt.Sprint(utils.CACHE_CHART_INFO_DATA, chartInfoId)
	return key
}

type ChartInfo struct {
	ChartInfoId     int       `gorm:"primaryKey;autoIncrement;column:chart_info_id"`
	ChartName       string    `gorm:"column:chart_name" description:"来源名称"`
	ChartNameEn     string    `gorm:"column:chart_name_en" description:"英文图表名称"`
	ChartClassifyId int       `gorm:"column:chart_classify_id" description:"图表分类id"`
	SysUserId       int       `gorm:"column:sys_user_id"`
	SysUserRealName string    `gorm:"column:sys_user_real_name"`
	UniqueCode      string    `gorm:"column:unique_code" description:"图表唯一编码"`
	CreateTime      time.Time `gorm:"column:create_time"`
	ModifyTime      time.Time `gorm:"column:modify_time"`
	DateType        int       `gorm:"column:date_type" description:"日期类型:1:00年至今,2:10年至今,3:15年至今，4：年初至今，5:自定义时间"`
	StartDate       string    `gorm:"column:start_date" description:"自定义开始日期"`
	EndDate         string    `gorm:"column:end_date" description:"自定义结束日期"`
	IsSetName       int       `gorm:"column:is_set_name" description:"设置名称"`
	EdbInfoIds      string    `gorm:"column:edb_info_ids" description:"指标id"`
	ChartType       int       `gorm:"column:chart_type" description:"生成样式:1:曲线图,2:季节性图,3:面积图，4：柱状图，5：散点图，6：组合图，7：柱方图，8：商品价格曲线图，9：相关性图"`
	Calendar        string    `gorm:"column:calendar" description:"公历/农历"`
	SeasonStartDate string    `gorm:"column:season_start_date" description:"季节性图开始日期"`
	SeasonEndDate   string    `gorm:"column:season_end_date" description:"季节性图开始日期（注意：这里可能是个错误，应该是结束日期）"`
	ChartImage      string    `gorm:"column:chart_image" description:"图表图片"`
	Sort            int       `gorm:"column:sort" description:"排序字段，数字越小越排前面"`
	LeftMin         string    `gorm:"column:left_min" description:"图表左侧最小值"`
	LeftMax         string    `gorm:"column:left_max" description:"图表左侧最大值"`
	RightMin        string    `gorm:"column:right_min" description:"图表右侧最小值"`
	RightMax        string    `gorm:"column:right_max" description:"图表右侧最大值"`
	Disabled        int       `gorm:"column:disabled" description:"是否禁用，0：启用，1：禁用，默认：0"`
	BarConfig       string    `gorm:"column:bar_config" description:"柱方图的配置，json数据"`
	Source          int       `gorm:"column:source" description:"1：ETA图库；2：商品价格曲线"`
	ExtraConfig     string    `gorm:"column:extra_config" description:"图表额外配置,json数据"`
}

// GetChartInfoById 通过id获取图表信息
func GetChartInfoById(chartInfoId int) (item *ChartInfo, err error) {
	sql := ` SELECT * FROM chart_info WHERE chart_info_id=? `
	err = global.DEFAULT_DmSQL.Raw(sql, chartInfoId).First(&item).Error
	return
}
